=begin
Lazy Array

Example
-----
b = ForwardArray.new
b <= [1, 2] + b

puts b.to_s
puts b[5]  => 2
=end

class Array
    def +(other)
        if other.is_a?(LazyArray)
            AddArray.new(self, other)
        else
            super
        end
    end
end

class LazyArray < Array
    # lazy array
    def +(other)
        if ! other.is_a?(LazyArray)
            other = LazyArray.new(other)
        end
        AddArray.new(self, other)
    end

end

class ForwardArray < LazyArray
    attr_accessor :subarray

    def initialize(subarray=[])
        @subarray = subarray
    end

    def [](k)
        @subarray[k]
    end

    def <=(other)
        @subarray = other
    end

    def to_s
        "ForwardArray"
    end
end

class AddArray < LazyArray
    <<~DOC
    An array, concatenating tow arrays with +

    Example:
    b = ForwardArray.new   # must define b here
    b <= [1, 2] + b  # b == [1, 2] + b i.e. b=[1,2,1,2,...]
    puts b[0]  # => 1
    puts b[5]  # => 2
    DOC

    attr_accessor :left, :right

    def initialize(left=[], right=[])
    <<~DOC
        Arguments
        ------
        left: left side of +
        right: right side of +
    DOC
        @left, @right = left, right
    end

    def [](k)
        l = @left.length
        if k < l
            return @left[k]
        else
            return @right[k - l]
        end
    end

    def to_s
        "#{@left} + #{@right}"
    end

end
